@@ -1,6 +1,4 @@
-/*  Copyright (C) 1988-2011 by Brian Doty and the 
-    Institute of Global Environment and Society (IGES).  
-    See file COPYRIGHT for more information.   */
+/* Copyright (C) 1988-2017 by George Mason University. See file COPYRIGHT for more information. */
 
 #ifndef DRIVER_GAGMAP
 #define WHERE extern
@@ -10,6 +8,7 @@
 
 WHERE FILE *gfile;
 WHERE FILE *mfile;
+WHERE FILE *exmfile;
 
 /* Following structures hold all the unpacked header info from a grib record. */
 
@@ -72,6 +71,8 @@ WHERE gaint scaneof;        /* option to ignore failure to find data at end of f
 WHERE gaint scanEOF;        /* option to ignore failure to find data at end of file */
 WHERE gaint scanlim;        /* the default # of max bytes between records */
 WHERE gaint notau;          /* force time to be base time */
+WHERE gaint upgrade;        /* create newer version of existing index file */
+WHERE gaint downgrade;      /* create older version of existing index file */
 WHERE gaint tauflg;         /* search for a fixed tau in filling the 4-D volume */
 WHERE gaint tauoff;         /* the fixed tau in h */
 WHERE gaint tau0;           /* set the base dtg for tau search */
@@ -113,6 +114,7 @@ extern void gribpr (struct grhdr *);
 /* function prototypes */
 gaint wtgmap(void) ;
 void  putint(gaint, unsigned char *,gaint *) ;
+gaint wtg1map(struct gafile *, struct gaindx *, struct gaindxb *);
 #if GRIB2
 void  g2fill (gaint, gaint, gaint, off_t, g2int, struct gag2indx *);
 gaint wtg2map (struct gafile *, struct gag2indx *);
